import JSZip from 'jszip';
import { AnalyzedElementType, ImportedElement, ImportElement, ShapeModificationCallback, SlideModificationCallback, SourceSlideIdentifier, StatusTracker } from '../types/types';
import { ISlide } from '../interfaces/islide';
import { IPresentationProps } from '../interfaces/ipresentation-props';
import { PresTemplate } from '../interfaces/pres-template';
import { RootPresTemplate } from '../interfaces/root-pres-template';
import { HelperElement } from '../types/xml-types';
import { ContentTracker } from '../helper/content-tracker';
export declare class Slide implements ISlide {
    /**
     * Source template of slide
     * @internal
     */
    sourceTemplate: PresTemplate;
    /**
     * Target template of slide
     * @internal
     */
    targetTemplate: RootPresTemplate;
    /**
     * Target number of slide
     * @internal
     */
    targetNumber: number;
    /**
     * Source number of slide
     * @internal
     */
    sourceNumber: number;
    /**
     * Target archive of slide
     * @internal
     */
    targetArchive: JSZip;
    /**
     * Source archive of slide
     * @internal
     */
    sourceArchive: JSZip;
    /**
     * Source path of slide
     * @internal
     */
    sourcePath: string;
    /**
     * Target path of slide
     * @internal
     */
    targetPath: string;
    /**
     * Modifications  of slide
     * @internal
     */
    modifications: SlideModificationCallback[];
    /**
     * Import elements of slide
     * @internal
     */
    importElements: ImportElement[];
    /**
     * Rels path of slide
     * @internal
     */
    relsPath: string;
    /**
     * Root template of slide
     * @internal
     */
    rootTemplate: RootPresTemplate;
    /**
     * Root  of slide
     * @internal
     */
    root: IPresentationProps;
    /**
     * Target rels path of slide
     * @internal
     */
    targetRelsPath: string;
    status: StatusTracker;
    content: ContentTracker;
    /**
     * List of unsupported tags in slide xml
     * @internal
     */
    unsupportedTags: string[];
    constructor(params: {
        presentation: IPresentationProps;
        template: PresTemplate;
        slideIdentifier: SourceSlideIdentifier;
    });
    /**
     * Try to convert a given slide's creationId to corresponding slide number.
     * Used if automizer is run with useCreationIds: true
     * @internal
     * @param PresTemplate
     * @slideNumber SourceSlideIdentifier
     * @returns number
     */
    getSlideNumber(template: PresTemplate, slideIdentifier: SourceSlideIdentifier): number;
    /**
     * Appends slide
     * @internal
     * @param targetTemplate
     * @returns append
     */
    append(targetTemplate: RootPresTemplate): Promise<void>;
    /**
     * Modifies slide
     * @internal
     * @param callback
     */
    modify(callback: SlideModificationCallback): void;
    /**
     * Adds slide to presentation
     * @internal
     * @returns slide to presentation
     */
    addSlideToPresentation(): Promise<void>;
    /**
     * Select and modify a single element on an added slide.
     * @param {string} selector - Element's name on the slide.
     * Should be a unique string defined on the "Selection"-pane within ppt.
     * @param {ShapeModificationCallback | ShapeModificationCallback[]} callback - One or more callback functions to apply.
     * Depending on the shape type (e.g. chart or table), different arguments will be passed to the callback.
     */
    modifyElement(selector: string, callback: ShapeModificationCallback | ShapeModificationCallback[]): this;
    /**
     * Select, insert and (optionally) modify a single element to a slide.
     * @param {string} presName - Filename or alias name of the template presentation.
     * Must have been importet with Automizer.load().
     * @param {number} slideNumber - Slide number within the specified template to search for the required element.
     * @param {ShapeModificationCallback | ShapeModificationCallback[]} callback - One or more callback functions to apply.
     * Depending on the shape type (e.g. chart or table), different arguments will be passed to the callback.
     */
    addElement(presName: string, slideNumber: number, selector: string, callback?: ShapeModificationCallback | ShapeModificationCallback[]): this;
    /**
     * Remove a single element from slide.
     * @param {string} selector - Element's name on the slide.
     */
    removeElement(selector: string): this;
    /**
     * Adds element to modifications list
     * @internal
     * @param presName
     * @param slideNumber
     * @param selector
     * @param mode
     * @param [callback]
     * @returns element to modifications list
     */
    private addElementToModificationsList;
    /**
     * Imported selected elements
     * @internal
     * @returns selected elements
     */
    importedSelectedElements(): Promise<void>;
    /**
     * Gets element info
     * @internal
     * @param importElement
     * @returns element info
     */
    getElementInfo(importElement: ImportElement): Promise<ImportedElement>;
    /**
     * Analyzes element
     * @internal
     * @param sourceElement
     * @param sourceArchive
     * @param slideNumber
     * @returns element
     */
    analyzeElement(sourceElement: XMLDocument, sourceArchive: JSZip, slideNumber: number): Promise<AnalyzedElementType>;
    /**
     * Applys modifications
     * @internal
     * @returns modifications
     */
    applyModifications(): Promise<void>;
    /**
     * Removes all unsupported tags from slide xml.
     * E.g. added relations & tags by Thinkcell cannot
     * be processed by pptx-automizer at the moment.
     * @internal
     */
    cleanSlide(): Promise<void>;
    /**
     * Copys slide files
     * @internal
     */
    copySlideFiles(): Promise<void>;
    /**
     * slideNote numbers differ from slide numbers if presentation
     * contains slides without notes. We need to find out
     * the proper enumeration of slideNote xml files.
     * @internal
     * @returns slide note file number
     */
    getSlideNoteSourceNumber(): Promise<number | undefined>;
    /**
     * Copys slide note files
     * @internal
     * @returns slide note files
     */
    copySlideNoteFiles(sourceNotesNumber: number): Promise<void>;
    /**
     * Updates slide note file
     * @internal
     * @returns slide note file
     */
    updateSlideNoteFile(sourceNotesNumber: number): Promise<void>;
    /**
     * Appends to slide rel
     * @internal
     * @param rootArchive
     * @param relId
     * @param slideCount
     * @returns to slide rel
     */
    appendToSlideRel(rootArchive: JSZip, relId: string, slideCount: number): Promise<HelperElement>;
    /**
     * Appends a new slide to slide list in presentation.xml.
     * If rootArchive has no slides, a new node will be created.
     * "id"-attribute of 'p:sldId'-element must be greater than 255.
     * @internal
     * @param rootArchive
     * @param relId
     * @returns to slide list
     */
    appendToSlideList(rootArchive: JSZip, relId: string): Promise<HelperElement>;
    /**
     * Appends slide to content type
     * @internal
     * @param rootArchive
     * @param slideCount
     * @returns slide to content type
     */
    appendSlideToContentType(rootArchive: JSZip, slideCount: number): Promise<HelperElement>;
    /**
     * Appends notes to content type
     * @internal
     * @param rootArchive
     * @param slideCount
     * @returns notes to content type
     */
    appendNotesToContentType(rootArchive: JSZip, slideCount: number): Promise<HelperElement>;
    /**
     * Copys related content
     * @internal
     * @returns related content
     */
    copyRelatedContent(): Promise<void>;
}
